@@ -1,10 +1,10 @@
-/* $Id: handletable.h 60692 2010-04-27 08:22:32Z umoeller $ */
+/* $Id: handletable.h 83687 2013-02-08 12:50:49Z fmehnert $ */
 /** @file
  * IPRT - Handle Tables, internal header.
  */
 
 /*
- * Copyright (C) 2008 Oracle Corporation
+ * Copyright (C) 2008-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -218,15 +218,12 @@ DECLINLINE(PRTHTENTRYCTX) rtHandleTableLookupWithCtx(PRTHANDLETABLEINT pThis, ui
  * Locks the handle table.
  *
  * @param   pThis           The handle table structure.
- * @param   pTmp            The spinlock temp variable.
  */
-DECLINLINE(void) rtHandleTableLock(PRTHANDLETABLEINT pThis, PRTSPINLOCKTMP pTmp)
+DECLINLINE(void) rtHandleTableLock(PRTHANDLETABLEINT pThis)
 {
     if (pThis->hSpinlock != NIL_RTSPINLOCK)
     {
-        RTSPINLOCKTMP const Tmp = RTSPINLOCKTMP_INITIALIZER;
-        *pTmp = Tmp;
-        RTSpinlockAcquire(pThis->hSpinlock, pTmp);
+        RTSpinlockAcquire(pThis->hSpinlock);
     }
 }
 
@@ -235,11 +232,10 @@ DECLINLINE(void) rtHandleTableLock(PRTHANDLETABLEINT pThis, PRTSPINLOCKTMP pTmp)
  * Locks the handle table.
  *
  * @param   pThis           The handle table structure.
- * @param   pTmp            The spinlock temp variable.
  */
-DECLINLINE(void) rtHandleTableUnlock(PRTHANDLETABLEINT pThis, PRTSPINLOCKTMP pTmp)
+DECLINLINE(void) rtHandleTableUnlock(PRTHANDLETABLEINT pThis)
 {
     if (pThis->hSpinlock != NIL_RTSPINLOCK)
-        RTSpinlockRelease(pThis->hSpinlock, pTmp);
+        RTSpinlockRelease(pThis->hSpinlock);
 }
 
